@@ -51,8 +51,8 @@ module Stable = struct
 
     let stable_witness (type a) : a Stable_witness.t -> a t Stable_witness.t =
       fun witness ->
-        let module Stable_witness = Stable_witness.Of_serializable1 (Format) (T) in
-        Stable_witness.of_serializable Format.stable_witness of_list_exn to_list witness
+      let module Stable_witness = Stable_witness.Of_serializable1 (Format) (T) in
+      Stable_witness.of_serializable Format.stable_witness of_list_exn to_list witness
     ;;
 
     let%expect_test _ =
@@ -153,13 +153,12 @@ module Stable = struct
   end
 end
 
-
 open Core
 module Unstable = Stable.V3
 
 module T' = struct
   type 'a t = 'a Stable.V3.t = ( :: ) of 'a * 'a list
-  [@@deriving compare, equal, hash, quickcheck, typerep, bin_io]
+  [@@deriving compare, equal, hash, quickcheck, typerep, bin_io, globalize]
 
   let sexp_of_t = Stable.V3.sexp_of_t
   let t_of_sexp = Stable.V3.t_of_sexp
@@ -185,8 +184,8 @@ module T' = struct
   let iter =
     `Custom
       (fun (hd :: tl) ~f ->
-         f hd;
-         List.iter tl ~f)
+        f hd;
+        List.iter tl ~f)
   ;;
 
   let iteri = `Define_using_fold
@@ -263,7 +262,7 @@ let filter_mapi (hd :: tl) ~f : _ list =
   let hd = f 0 hd in
   let[@inline always] f i x = f (i + 1) x in
   match hd with
-  | None -> List.filter_mapi tl ~f
+  | None -> List.filter_mapi tl ~f [@nontail]
   | Some hd -> hd :: List.filter_mapi tl ~f
 ;;
 
@@ -277,11 +276,19 @@ let filteri (hd :: tl) ~f : _ list =
   let include_hd = f 0 hd in
   let[@inline always] f i x = f (i + 1) x in
   match include_hd with
-  | false -> List.filteri tl ~f
+  | false -> List.filteri tl ~f [@nontail]
   | true -> hd :: List.filteri tl ~f
 ;;
 
-let map t ~f = mapi t ~f:(fun (_ : int) x -> f x)
+let map t ~f = mapi t ~f:(fun (_ : int) x -> f x) [@nontail]
+
+let map2 t1 t2 ~f : _ List.Or_unequal_lengths.t =
+  match List.map2 (to_list t1) (to_list t2) ~f with
+  | Ok x -> Ok (of_list_exn x)
+  | Unequal_lengths -> Unequal_lengths
+;;
+
+let map2_exn t1 t2 ~f = List.map2_exn (to_list t1) (to_list t2) ~f |> of_list_exn
 let reduce (hd :: tl) ~f = List.fold ~init:hd tl ~f
 
 let reverse (hd :: tl) =
@@ -295,17 +302,17 @@ let reverse (hd :: tl) =
 
 let append (hd :: tl) l = hd :: List.append tl l
 
-include Monad.Make (struct
-    type nonrec 'a t = 'a t
+include Monad.Make_local (struct
+  type nonrec 'a t = 'a t
 
-    let return hd = [ hd ]
-    let map = `Custom map
+  let return hd = [ hd ]
+  let map = `Custom map
 
-    let bind (hd :: tl) ~f =
-      let f_hd = f hd in
-      append f_hd (List.concat_map tl ~f:(fun x -> to_list (f x)))
-    ;;
-  end)
+  let bind (hd :: tl) ~f =
+    let f_hd = f hd in
+    append f_hd (List.concat_map tl ~f:(fun x -> to_list (f x)))
+  ;;
+end)
 
 let unzip ((hd1, hd2) :: tl) =
   let tl1, tl2 = List.unzip tl in
@@ -338,13 +345,15 @@ let to_sequence t =
 
 let sort t ~compare = List.sort (to_list t) ~compare |> of_list_exn
 let stable_sort t ~compare = List.stable_sort (to_list t) ~compare |> of_list_exn
-let dedup_and_sort ~compare t = List.dedup_and_sort ~compare (to_list t) |> of_list_exn
+let dedup_and_sort t ~compare = List.dedup_and_sort ~compare (to_list t) |> of_list_exn
+let permute ?random_state t = List.permute ?random_state (to_list t) |> of_list_exn
 
 let min_elt' (hd :: tl) ~compare =
-  List.fold tl ~init:hd ~f:(fun min elt -> if compare min elt > 0 then elt else min)
+  List.fold tl ~init:hd ~f:(fun min elt -> if compare min elt > 0 then elt else min) [@nontail
+                                                                                       ]
 ;;
 
-let max_elt' t ~compare = min_elt' t ~compare:(fun x y -> compare y x)
+let max_elt' t ~compare = min_elt' t ~compare:(fun x y -> compare y x) [@nontail]
 
 let map_add_multi map ~key ~data =
   Map.update map key ~f:(function
@@ -359,6 +368,13 @@ let map_of_container_multi fold container ~comparator =
 
 let map_of_alist_multi alist = map_of_container_multi List.fold alist
 let map_of_sequence_multi sequence = map_of_container_multi Sequence.fold sequence
+let fold_nonempty (hd :: tl) ~init ~f = List.fold tl ~init:(init hd) ~f
+
+let map_of_list_with_key_multi list ~comparator ~get_key =
+  List.fold list ~init:(Map.empty comparator) ~f:(fun acc data ->
+    let key = get_key data in
+    map_add_multi acc ~key ~data)
+;;
 
 let fold_right (hd :: tl) ~init:acc ~f =
   let acc = List.fold_right tl ~init:acc ~f in
@@ -413,6 +429,10 @@ let init n ~f =
   hd :: tl
 ;;
 
+let cartesian_product t t' =
+  List.cartesian_product (to_list t) (to_list t') |> of_list_exn
+;;
+
 module Reversed = struct
   type 'a t = ( :: ) of 'a * 'a Reversed_list.t
 
@@ -427,7 +447,7 @@ module Reversed = struct
   ;;
 
   let rev_mapi (hd :: tl : _ t) ~f = rev_map_aux 1 tl ~f ([ f 0 hd ] : _ T'.t)
-  let rev_map t ~f = rev_mapi t ~f:(fun _ x -> f x)
+  let rev_map t ~f = rev_mapi t ~f:(fun _ x -> f x) [@nontail]
   let cons x t = x :: to_rev_list t
 
   module With_sexp_of = struct
