@@ -6,21 +6,21 @@ module Heap = Pairing_heap
 (* Container tests.  Heap has no t_of_sexp because there is no way to deserialize a
    comparison function, so we simulate it. *)
 include Base_test_helpers.Test_container.Test_S0 (struct
-    include Heap
+  include Heap
 
-    module Elt = struct
-      type t = int [@@deriving sexp]
+  module Elt = struct
+    type t = int [@@deriving sexp]
 
-      let of_int = Fn.id
-      let to_int = Fn.id
-    end
+    let of_int = Fn.id
+    let to_int = Fn.id
+  end
 
-    type nonrec t = int t [@@deriving sexp_of]
+  type nonrec t = int t [@@deriving sexp_of]
 
-    let mem t int = mem t int ~equal:Int.equal
-    let of_list ints = of_list ints ~cmp:Int.compare
-    let t_of_sexp sexp = sexp |> [%of_sexp: int list] |> of_list
-  end)
+  let mem t int = mem t int ~equal:Int.equal
+  let of_list ints = of_list ints ~cmp:Int.compare
+  let t_of_sexp sexp = sexp |> [%of_sexp: int list] |> of_list
+end)
 
 let%expect_test "Heap.sexp_of_t" =
   let test list =
@@ -38,13 +38,15 @@ let%expect_test "Heap.sexp_of_t" =
   test [ 3 ];
   [%expect {|
     (3)
-    () |}];
+    ()
+    |}];
   test [ 3; 1; 4 ];
   [%expect {|
     (1 3 4)
     (3 4)
     (4)
-    () |}];
+    ()
+    |}];
   test [ 3; 1; 4; 1; 5; 9; 2; 6; 5 ];
   [%expect
     {|
@@ -57,7 +59,8 @@ let%expect_test "Heap.sexp_of_t" =
     (5 6 9)
     (6 9)
     (9)
-    () |}]
+    ()
+    |}]
 ;;
 
 let%expect_test "Heap.sexp_of_t with removes" =
@@ -72,18 +75,19 @@ let%expect_test "Heap.sexp_of_t with removes" =
       print_s [%sexp (heap : int Heap.t)])
   in
   test [];
-  [%expect {|
-    () |}];
+  [%expect {| () |}];
   test [ 3 ];
   [%expect {|
     (3)
-    () |}];
+    ()
+    |}];
   test [ 3; 1; 4 ];
   [%expect {|
     (1 3 4)
     (1 4)
     (4)
-    () |}];
+    ()
+    |}];
   test [ 3; 1; 4; 1; 5; 9; 2; 6; 5 ];
   [%expect
     {|
@@ -96,7 +100,8 @@ let%expect_test "Heap.sexp_of_t with removes" =
     (2 5 6)
     (5 6)
     (5)
-    () |}]
+    ()
+    |}]
 ;;
 
 open! Heap
@@ -281,7 +286,7 @@ let integers n =
   for i = 1 to n do
     add t i;
     if i % 10 = 0
-    (* We need to pop from time to time to trigger the amortized tree reorganizations.  If
+       (* We need to pop from time to time to trigger the amortized tree reorganizations.  If
        we don't do this the resulting structure is just a linked list and the caller is
        not flexed as completely as it should be. *)
     then (
@@ -395,3 +400,57 @@ let%expect_test "pop_if on empty heap" =
   let empty = Heap.create ~cmp:Int.compare () in
   require_none [%here] [%sexp_of: int] (Heap.pop_if empty (fun _ -> true))
 ;;
+
+let test_pop_while_negative values =
+  let heap = Heap.of_list values ~cmp:Int.compare in
+  print_s [%message "before" (heap : int Heap.t)];
+  let popped = Heap.pop_while heap (fun i -> i < 0) in
+  print_s [%message "after" (popped : int list) (heap : int Heap.t)]
+;;
+
+let%expect_test "pop_while single" =
+  test_pop_while_negative [ -1; 1; 2; 3 ];
+  [%expect
+    {|
+    (before (heap (-1 1 2 3)))
+    (after (popped (-1)) (heap (1 2 3)))
+    |}]
+;;
+
+let%expect_test "pop_while empty" =
+  test_pop_while_negative [];
+  [%expect
+    {|
+    (before (heap ()))
+    (after
+      (popped ())
+      (heap   ()))
+    |}]
+;;
+
+let%expect_test "pop_while no popped" =
+  test_pop_while_negative [ 5; 1; 10; 3; 2 ];
+  [%expect
+    {|
+    (before (heap (1 2 3 5 10)))
+    (after (popped ()) (heap (1 2 3 5 10)))
+    |}]
+;;
+
+let%expect_test "pop_while all popped" =
+  test_pop_while_negative [ -10; -1; -3; -2 ];
+  [%expect
+    {|
+    (before (heap (-10 -3 -2 -1)))
+    (after (popped (-10 -3 -2 -1)) (heap ()))
+    |}]
+;;
+
+let%expect_test "pop_while repeated" =
+  test_pop_while_negative [ -10; 2; -1; -1; -1; 3; 10 ];
+  [%expect
+    {|
+    (before (heap (-10 -1 -1 -1 2 3 10)))
+    (after (popped (-10 -1 -1 -1)) (heap (2 3 10)))
+    |}]
+;;
