@@ -237,7 +237,7 @@ module Config = struct
     ; level_bits : Level_bits.t [@default level_bits_default]
     ; capacity : int option [@sexp.option]
     }
-  [@@deriving fields, sexp]
+  [@@deriving fields ~getters ~iterators:iter, sexp]
 
   let alarm_precision t = Alarm_precision.to_span t.alarm_precision
 
@@ -295,7 +295,6 @@ module Config = struct
   ;;
 end
 
-
 (** Timing wheel is implemented as a priority queue in which the keys are
     non-negative integers corresponding to the intervals of time.  The priority queue is
     unlike a typical priority queue in that rather than having a "delete min" operation,
@@ -522,7 +521,6 @@ end = struct
   module Slots_mask = Key.Slots_mask
 
   module External_elt = struct
-
     (** The [pool_slots] here has nothing to do with the slots in a level array.  This is
         for the slots in the pool tuple representing a level element. *)
     type 'a pool_slots =
@@ -532,7 +530,7 @@ end = struct
       , int
       , 'a pool_slots Pointer.t
       , 'a pool_slots Pointer.t )
-        Pool.Slots.t6
+      Pool.Slots.t6
     [@@deriving sexp_of]
 
     type 'a t = 'a pool_slots Pointer.t [@@deriving sexp_of]
@@ -569,7 +567,7 @@ end = struct
     val create
       :  'a Pool.t
       -> key:Key.t
-      (** [at] is used when the priority queue is used to implement a timing wheel.  If
+           (** [at] is used when the priority queue is used to implement a timing wheel.  If
           unused, it will be [Time_ns.epoch]. *)
       -> at:Time_ns.t
       -> value:'a
@@ -778,7 +776,7 @@ end = struct
            of elts.  [Array.length slots = 1 lsl bits]. *)
         slots : ('a Internal_elt.t array[@sexp.opaque])
       }
-    [@@deriving fields, sexp_of]
+    [@@deriving fields ~getters ~iterators:iter, sexp_of]
 
     let slot t ~key = Key.slot key ~bits_per_slot:t.bits_per_slot ~slots_mask:t.slots_mask
     let next_slot t slot = Slots_mask.next_slot t.slots_mask slot
@@ -805,7 +803,7 @@ end = struct
       mutable elt_key_lower_bound : Key.t
     ; levels : 'a Level.t array
     }
-  [@@deriving fields, sexp_of]
+  [@@deriving fields ~getters ~iterators:iter, sexp_of]
 
   type 'a priority_queue = 'a t
 
@@ -868,10 +866,10 @@ end = struct
         (let r = ref [] in
          internal_iter t ~f:(fun elt ->
            r
-           := { Pretty.Elt.key = Internal_elt.key pool elt
-              ; value = Internal_elt.value pool elt
-              }
-              :: !r);
+             := { Pretty.Elt.key = Internal_elt.key pool elt
+                ; value = Internal_elt.value pool elt
+                }
+                :: !r);
          List.rev !r)
     }
   ;;
@@ -927,9 +925,9 @@ end = struct
                assert (
                  length
                  = Array.fold level.slots ~init:0 ~f:(fun n elt ->
-                   if Internal_elt.is_null elt
-                   then n
-                   else n + Internal_elt.length pool elt))))
+                     if Internal_elt.is_null elt
+                     then n
+                     else n + Internal_elt.length pool elt))))
           ~min_allowed_key:
             (check (fun min_allowed_key ->
                assert (Key.( >= ) min_allowed_key Key.zero);
@@ -1174,11 +1172,11 @@ end = struct
       elements are encountered, they are removed from the timing wheel if their key is
       smaller than [t_min_allowed_key], or added at a lower level if not. *)
   let increase_level_min_allowed_key
-        t
-        (level : _ Level.t)
-        ~prev_level_max_allowed_key
-        ~t_min_allowed_key
-        ~handle_removed
+    t
+    (level : _ Level.t)
+    ~prev_level_max_allowed_key
+    ~t_min_allowed_key
+    ~handle_removed
     =
     let desired_min_allowed_key =
       Level.compute_min_allowed_key level ~prev_level_max_allowed_key
@@ -1215,7 +1213,7 @@ end = struct
     done;
     level.min_allowed_key <- desired_min_allowed_key;
     level.max_allowed_key
-    <- Key.add_clamp_to_max desired_min_allowed_key level.diff_max_min_allowed_key
+      <- Key.add_clamp_to_max desired_min_allowed_key level.diff_max_min_allowed_key
   ;;
 
   module Increase_min_allowed_key_result = struct
@@ -1280,37 +1278,37 @@ end = struct
             (bits_per_slot, max_level_min_allowed_key, levels)
             (level_bits : Num_key_bits.t)
             ->
-              let keys_per_slot = Key.num_keys bits_per_slot in
-              let diff_max_min_allowed_key =
-                compute_diff_max_min_allowed_key ~level_bits ~bits_per_slot
-              in
-              let min_key_in_same_slot_mask = Min_key_in_same_slot_mask.create ~bits_per_slot in
-              let min_allowed_key =
-                Key.min_key_in_same_slot max_level_min_allowed_key min_key_in_same_slot_mask
-              in
-              let max_allowed_key =
-                Key.add_clamp_to_max min_allowed_key diff_max_min_allowed_key
-              in
-              let level =
-                { Level.index
-                ; bits = level_bits
-                ; slots_mask = Slots_mask.create ~level_bits
-                ; bits_per_slot
-                ; keys_per_slot
-                ; min_key_in_same_slot_mask
-                ; diff_max_min_allowed_key
-                ; length = 0
-                ; min_allowed_key
-                ; max_allowed_key
-                ; slots =
-                    Array.create
-                      ~len:(Int63.to_int_exn (Num_key_bits.pow2 level_bits))
-                      (Internal_elt.null ())
-                }
-              in
-              ( Num_key_bits.( + ) level_bits bits_per_slot
-              , Key.succ_clamp_to_max max_allowed_key
-              , level :: levels ))
+        let keys_per_slot = Key.num_keys bits_per_slot in
+        let diff_max_min_allowed_key =
+          compute_diff_max_min_allowed_key ~level_bits ~bits_per_slot
+        in
+        let min_key_in_same_slot_mask = Min_key_in_same_slot_mask.create ~bits_per_slot in
+        let min_allowed_key =
+          Key.min_key_in_same_slot max_level_min_allowed_key min_key_in_same_slot_mask
+        in
+        let max_allowed_key =
+          Key.add_clamp_to_max min_allowed_key diff_max_min_allowed_key
+        in
+        let level =
+          { Level.index
+          ; bits = level_bits
+          ; slots_mask = Slots_mask.create ~level_bits
+          ; bits_per_slot
+          ; keys_per_slot
+          ; min_key_in_same_slot_mask
+          ; diff_max_min_allowed_key
+          ; length = 0
+          ; min_allowed_key
+          ; max_allowed_key
+          ; slots =
+              Array.create
+                ~len:(Int63.to_int_exn (Num_key_bits.pow2 level_bits))
+                (Internal_elt.null ())
+          }
+        in
+        ( Num_key_bits.( + ) level_bits bits_per_slot
+        , Key.succ_clamp_to_max max_allowed_key
+        , level :: levels ))
     in
     { length = 0
     ; pool = Internal_elt.Pool.create ?capacity ()
@@ -1327,7 +1325,7 @@ end = struct
     if Internal_elt.equal elt t.min_elt
     then
       t.min_elt <- Internal_elt.null ()
-    (* We keep [t.elt_lower_bound] since it is valid even though [t.min_elt] is being
+      (* We keep [t.elt_lower_bound] since it is valid even though [t.min_elt] is being
        removed. *);
     t.length <- t.length - 1;
     let level = t.levels.(Internal_elt.level_index pool elt) in
@@ -1429,7 +1427,7 @@ type 'a t =
   ; mutable max_allowed_alarm_time : Time_ns.t
   ; priority_queue : 'a Priority_queue.t
   }
-[@@deriving fields, sexp_of]
+[@@deriving fields ~getters ~iterators:iter, sexp_of]
 
 type 'a timing_wheel = 'a t
 type 'a t_now = 'a t
@@ -1455,7 +1453,7 @@ module Pretty = struct
       { at : Time_ns.t
       ; value : 'a
       }
-    [@@deriving fields, sexp_of]
+    [@@deriving fields ~getters, sexp_of]
 
     let create t alarm = { at = Alarm.at t alarm; value = Alarm.value t alarm }
     let compare t1 t2 = Time_ns.compare (at t1) (at t2)
@@ -1472,14 +1470,14 @@ module Pretty = struct
 end
 
 let pretty
-      ({ config
-       ; start
-       ; max_interval_num
-       ; now
-       ; now_interval_num_start = _
-       ; max_allowed_alarm_time = _
-       ; priority_queue = _
-       } as t)
+  ({ config
+   ; start
+   ; max_interval_num
+   ; now
+   ; now_interval_num_start = _
+   ; max_allowed_alarm_time = _
+   ; priority_queue = _
+   } as t)
   =
   let r = ref [] in
   iter t ~f:(fun a -> r := Pretty.Alarm.create t a :: !r);
@@ -1670,6 +1668,23 @@ let advance_clock t ~to_ ~handle_fired =
       t.max_allowed_alarm_time <- compute_max_allowed_alarm_time t)
 ;;
 
+let advance_clock_stop_at_next_alarm t ~to_ ~handle_fired =
+  let min_elt = Priority_queue.min_elt_ t.priority_queue in
+  if Internal_elt.is_null min_elt
+  then advance_clock t ~to_ ~handle_fired:(fun _ -> assert false)
+  else (
+    let key = Internal_elt.key (pool t) min_elt in
+    (* as an optimization, compare against [interval_num_start] to avoid the potentially
+       costly computation of [Internal_elt.min_alarm_time] *)
+    if Time_ns.( < ) to_ (interval_num_start t key)
+    then advance_clock t ~to_ ~handle_fired:(fun _ -> assert false)
+    else (
+      let to_ =
+        Time_ns.min to_ (Internal_elt.min_alarm_time (pool t) min_elt ~with_key:key)
+      in
+      advance_clock t ~to_ ~handle_fired))
+;;
+
 let create ~config ~start =
   if Time_ns.( < ) start Time_ns.epoch
   then
