@@ -5,18 +5,18 @@ module With_integer_index = struct
   module Kernel : sig
     type 'a t
 
-    val length : (_ t[@local]) -> int
+    val length : _ t -> int
     val capacity : _ t -> int
     val create : ?initial_capacity:int -> unit -> _ t
     val unsafe_create_uninitialized : len:int -> 'a t
-    val init : int -> f:((int -> 'a)[@local]) -> 'a t
+    val init : int -> f:(int -> 'a) -> 'a t
     val unsafe_get : 'a t -> int -> 'a
     val unsafe_set : 'a t -> int -> 'a -> unit
 
     val unsafe_blit
-      :  src:('a t[@local])
+      :  src:'a t
       -> src_pos:int
-      -> dst:('a t[@local])
+      -> dst:'a t
       -> dst_pos:int
       -> len:int
       -> unit
@@ -30,6 +30,10 @@ module With_integer_index = struct
     val copy : 'a t -> 'a t
     val sort : ?pos:int -> ?len:int -> 'a t -> compare:('a -> 'a -> int) -> unit
 
+    module Expert : sig
+      val unsafe_inner : 'a t -> Obj.t Uniform_array.t
+    end
+
     module With_structure_details : sig
       type nonrec 'a t = 'a t [@@deriving sexp_of]
     end
@@ -38,10 +42,10 @@ module With_integer_index = struct
       { mutable arr : Obj.t Uniform_array.t
       ; mutable length : int
       ; mutable capacity : int
-      (** Invariant: [capacity = Uniform_array.length arr].
+          (** Invariant: [capacity = Uniform_array.length arr].
           We maintain it here to eliminate an indirection when accessing long arrays. *)
       }
-    [@@deriving fields]
+    [@@deriving fields ~getters ~setters]
 
     let length t = t.length
 
@@ -168,6 +172,10 @@ module With_integer_index = struct
       in
       Uniform_array.sort ~pos ~len t.arr ~compare
     ;;
+
+    module Expert = struct
+      let[@inline always] unsafe_inner t = t.arr
+    end
   end
 
   include Kernel
@@ -207,6 +215,10 @@ module With_integer_index = struct
 
   let maybe_get t i = if i < 0 || i >= length t then None else Some (unsafe_get t i)
 
+  let maybe_get_local t i =
+    if i < 0 || i >= length t then None else Some { Gel.g = unsafe_get t i }
+  ;;
+
   let set t i element =
     check_index t i ~op:"set";
     unsafe_set t i element
@@ -264,16 +276,32 @@ module With_integer_index = struct
     e
   ;;
 
-  let grow_to t ~len ~default =
+  let[@inline never] grow_to_unchecked t ~len ~default =
+    grow_capacity_to_at_least t ~capacity:len;
+    for i = length t to len - 1 do
+      unsafe_set t i default
+    done;
+    set_length t len
+  ;;
+
+  let[@inline always] grow_to t ~len ~default =
+    if len > length t then grow_to_unchecked t ~len ~default
+  ;;
+
+  let grow_to_include t idx ~default = grow_to t ~len:(idx + 1) ~default
+
+  let grow_to' t ~len ~default =
     if len > length t
     then (
       grow_capacity_to_at_least t ~capacity:len;
       for i = length t to len - 1 do
-        unsafe_set t i default
+        unsafe_set t i (default i)
       done;
       set_length t len)
   ;;
 
+  let grow_to_include' t idx ~default = grow_to' t ~len:(idx + 1) ~default
+
   let shrink_to t ~len =
     if len < 0
     then raise__bad_index t len ~op:"shrink_to"
@@ -285,6 +313,13 @@ module With_integer_index = struct
       set_length t len)
   ;;
 
+  let shrink_to_imm (t : 'a t) (_ : 'a Type_immediacy.Always.t) ~len =
+    if len < 0
+    then raise__bad_index t len ~op:"shrink_to_imm"
+    else if len < length t
+    then set_length t len
+  ;;
+
   let iteri t ~f =
     for i = 0 to max_index t do
       f i (unsafe_get t i)
@@ -305,6 +340,11 @@ module With_integer_index = struct
     !result
   ;;
 
+  let to_local_list t =
+    let rec aux t i acc = if i < 0 then acc else aux t (i - 1) (unsafe_get t i :: acc) in
+    aux t (max_index t) []
+  ;;
+
   let to_alist t =
     let result = ref [] in
     for i = max_index t downto 0 do
@@ -313,6 +353,15 @@ module With_integer_index = struct
     !result
   ;;
 
+  (* Convert to a sequence but does not attempt to protect against modification
+     in the vec. *)
+  let to_sequence_mutable t =
+    Sequence.unfold_step ~init:0 ~f:(fun i ->
+      if i >= length t then Done else Yield { value = unsafe_get t i; state = i + 1 })
+  ;;
+
+  let to_sequence t = to_sequence_mutable (copy t)
+
   let of_list xs =
     let t = create ~initial_capacity:(List.length xs) () in
     List.iter xs ~f:(push_back t);
@@ -321,6 +370,21 @@ module With_integer_index = struct
 
   let of_array arr = init (Array.length arr) ~f:(fun i -> Array.get arr i)
 
+  let of_sequence seq =
+    (* We don't know the length of seq, so we can't set an initial capacity *)
+    let t = create () in
+    Sequence.iter seq ~f:(push_back t);
+    t
+  ;;
+
+  let foldi t ~init ~f =
+    let r = ref init in
+    for i = 0 to max_index t do
+      r := f i !r (unsafe_get t i)
+    done;
+    !r
+  ;;
+
   let fold t ~init ~f =
     let r = ref init in
     for i = 0 to max_index t do
@@ -329,21 +393,32 @@ module With_integer_index = struct
     !r
   ;;
 
+  let foldi_local_accum t ~init:acc ~f =
+    let rec aux t i ~acc ~f =
+      if i >= length t
+      then acc
+      else (
+        let acc = f i acc (unsafe_get t i) in
+        aux t (i + 1) ~acc ~f)
+    in
+    aux t 0 ~acc ~f
+  ;;
+
   include Blit.Make1 (struct
-      type nonrec 'a t = 'a t
+    type nonrec 'a t = 'a t
 
-      let create_like ~len _t =
-        (* Note that even though we [unsafe_create_uninitialized], every time this function
+    let create_like ~len _t =
+      (* Note that even though we [unsafe_create_uninitialized], every time this function
            is called, the [Vec] is immediately blitted with valid values. *)
-        Kernel.unsafe_create_uninitialized ~len
-      ;;
+      Kernel.unsafe_create_uninitialized ~len
+    ;;
 
-      let length = length
-      let unsafe_blit = unsafe_blit
-    end)
+    let length = length
+    let unsafe_blit = unsafe_blit
+  end)
 
   (** Returns the length of the longest prefix for which [f] is true. *)
-  let take_while_len t ~f:(f [@local]) =
+  let take_while_len t ~f =
     let rec loop i =
       if i >= length t || not (f (get t i)) then i else (loop [@tailcall]) (i + 1)
     in
@@ -409,6 +484,10 @@ module With_integer_index = struct
 
   let clear t = if length t > 0 then shrink_to t ~len:0
 
+  let clear_imm (t : 'a t) (proof : 'a Type_immediacy.Always.t) =
+    if length t > 0 then shrink_to_imm t proof ~len:0
+  ;;
+
   let sexp_of_t (type a) (sexp_of_a : a -> Sexp.t) t =
     let t = to_list t in
     [%sexp (t : a list)]
@@ -585,18 +664,18 @@ module With_integer_index = struct
       type nonrec 'a t = 'a t [@@deriving compare, sexp]
 
       include Bin_prot.Utils.Make_iterable_binable1 (struct
-          type nonrec 'a t = 'a t
-          type 'a el = 'a [@@deriving bin_io]
+        type nonrec 'a t = 'a t
+        type 'a el = 'a [@@deriving bin_io]
 
-          let caller_identity =
-            Bin_prot.Shape.Uuid.of_string "2ec1d047-7cf8-49bc-991b-0badd17d8359"
-          ;;
+        let caller_identity =
+          Bin_prot.Shape.Uuid.of_string "2ec1d047-7cf8-49bc-991b-0badd17d8359"
+        ;;
 
-          let module_name = Some "Vec"
-          let init ~len ~next = init len ~f:(fun _ -> next ())
-          let iter = iter
-          let length = length
-        end)
+        let module_name = Some "Vec"
+        let init ~len ~next = init len ~f:(fun _ -> next ())
+        let iter = iter
+        let length = length
+      end)
     end
   end
 end
@@ -611,10 +690,25 @@ module Make (M : Intable.S) = struct
   let[@inline always] unsafe_get t index = unsafe_get t (M.to_int_exn index)
   let get t index = get t (M.to_int_exn index)
   let maybe_get t index = maybe_get t (M.to_int_exn index)
+  let maybe_get_local t index = maybe_get_local t (M.to_int_exn index)
   let[@inline always] unsafe_set t index = unsafe_set t (M.to_int_exn index)
   let set t index = set t (M.to_int_exn index)
   let next_free_index t = next_free_index t |> M.of_int_exn
-  let iteri t ~f = iteri t ~f:(fun [@inline] int x -> f (M.of_int_exn int) x) [@nontail]
+
+  let foldi t ~init ~f =
+    (foldi [@inlined hint]) t ~init ~f:(fun [@inline] int accum x ->
+      f (M.of_int_exn int) accum x) [@nontail]
+  ;;
+
+  let foldi_local_accum t ~init ~f =
+    (foldi_local_accum [@inlined hint]) t ~init ~f:(fun [@inline] int accum x ->
+      f (M.of_int_exn int) accum x) [@nontail]
+  ;;
+
+  let iteri t ~f =
+    (iteri [@inlined hint]) t ~f:(fun [@inline] int x -> f (M.of_int_exn int) x) [@nontail]
+  ;;
+
   let push_back_index t element = push_back_index t element |> M.of_int_exn
 
   let to_alist t =
@@ -634,6 +728,17 @@ module Make (M : Intable.S) = struct
     !result
   ;;
 
+  let grow_to_include t idx ~default = grow_to_include t (M.to_int_exn idx) ~default
+
+  let grow_to' t ~len ~default =
+    grow_to' t ~len ~default:(fun [@inline] idx -> default (M.of_int_exn idx))
+  ;;
+
+  let grow_to_include' t idx ~default =
+    grow_to_include' t (M.to_int_exn idx) ~default:(fun [@inline] idx ->
+      default (M.of_int_exn idx))
+  ;;
+
   module Inplace = struct
     include Inplace
 
